"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .principal import Principal, PrincipalTypedDict
from acuvity.types import BaseModel
from datetime import datetime
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class AlerteventTypedDict(TypedDict):
    r"""Represents an alert event raised by a policy."""

    alert_definition: str
    r"""The name of the alert definition that triggered the alert event."""
    principal: PrincipalTypedDict
    r"""Describe the principal."""
    alert_definition_namespace: NotRequired[str]
    r"""The namespace of the alert definition."""
    provider: NotRequired[str]
    r"""The provider used that the alert came from."""
    timestamp: NotRequired[datetime]
    r"""When the alert event was raised."""


class Alertevent(BaseModel):
    r"""Represents an alert event raised by a policy."""

    alert_definition: Annotated[str, pydantic.Field(alias="alertDefinition")]
    r"""The name of the alert definition that triggered the alert event."""

    principal: Principal
    r"""Describe the principal."""

    alert_definition_namespace: Annotated[
        Optional[str], pydantic.Field(alias="alertDefinitionNamespace")
    ] = None
    r"""The namespace of the alert definition."""

    provider: Optional[str] = None
    r"""The provider used that the alert came from."""

    timestamp: Optional[datetime] = None
    r"""When the alert event was raised."""



# pylint: disable=protected-access

import os
import base64
from typing import Any, Dict, Iterable, List, Optional, Sequence, Tuple, Union
from acuvity.models import Analyzer, Extractionrequest, Policerequest, Policeresponse, Scanrequest, Scanresponse, ScanrequestType, Type, ScanrequestAnonymization, Anonymization, Policeexternaluser
from acuvity.sdkconfiguration import SDKConfiguration

from .apex import Apex

class ApexExtended(Apex):
    def __init__(self, sdk_config: SDKConfiguration) -> None:
        super().__init__(sdk_config)
        self._available_analyzers: Optional[List[Analyzer]] = None

    def list_analyzer_groups(self) -> List[str]:
        """
        list_analyzer_groups() returns a list of all available analyzer groups. These can be passed in a scan request
        to activate/deactivate a whole group of analyzers at once.

        NOTE: this call is cached for the lifetime of the SDK object.
        """
        if self._available_analyzers is None:
            self._available_analyzers = self.list_analyzers()
        return sorted({ a.group for a in self._available_analyzers if a.group is not None })

    def list_analyzer_names(self, group: Optional[str] = None) -> List[str]:
        """
        list_analyzer_names() returns a list of all available analyzer names. These can be passed in a scan request
        to activate/deactivate specific analyzers.

        :param group: the group of analyzers to filter the list by. If not provided, all analyzers will be returned.

        NOTE: this call is cached for the lifetime of the SDK object.
        """
        if self._available_analyzers is None:
            self._available_analyzers = self.list_analyzers()
        return sorted([ a.id for a in self._available_analyzers if (group is None or a.group == group) and a.id is not None ])

    def scan(
        self,
        *messages: str,
        files: Union[Sequence[Union[str,os.PathLike]], os.PathLike, str, None] = None,
        request_type: Union[ScanrequestType,str] = ScanrequestType.INPUT,
        annotations: Optional[Dict[str, str]] = None,
        analyzers: Optional[List[str]] = None,
        bypass_hash: Optional[str] = None,
        anonymization: Union[ScanrequestAnonymization, str, None] = None,
        redactions: Optional[List[str]] = None,
        keywords: Optional[List[str]] = None,
        access_policy: Optional[str] = None,
        content_policy: Optional[str] = None,
    ) -> Scanresponse:
        """
        scan() runs the provided messages (prompts) through the Acuvity detection engines and returns the results. Alternatively, you can run model output through the detection engines.
        Returns a Scanresponse object on success, and raises different exceptions on failure.

        This function allows to use and try different analyzers and make use of the redaction feature.
        You can also run access policies and content policies by passing them as parameters.

        :param messages: the messages to scan. These are the prompts that you want to scan. Required if no files or a direct request object are provided.
        :param files: the files to scan. These are the files that you want to scan. Required if no messages are provided. Can be used in addition to messages.
        :param request_type: the type of the validation. This can be either ScanrequestType.INPUT or ScanrequestType.OUTPUT. Defaults to ScanrequestType.INPUT. Use ScanrequestType.OUTPUT if you want to run model output through the detection engines.
        :param annotations: the annotations to use. These are the annotations that you want to use. If not provided, no annotations will be used.
        :param analyzers: the analyzers to use. These are the analyzers that you want to use. If not provided, the internal default analyzers will be used. Use "+" to include an analyzer and "-" to exclude an analyzer. For example, ["+image-classifier", "-modality-detector"] will include the image classifier and exclude the modality detector. If any analyzer does not start with a '+' or '-', then the default analyzers will be replaced by whatever is provided. Call `list_analyzers()` and/or its variants to get a list of available analyzers.
        :param bypass_hash: the bypass hash to use. This is the hash that you want to use to bypass the detection engines. If not provided, no bypass hash will be used.
        :param anonymization: the anonymization to use. This is the anonymization that you want to use. If not provided, but the returned detections contain redactions, then the system will use the internal defaults for anonymization which is subject to change.
        :param redactions: the redactions to apply. If your want to redact certain parts of the returned detections, you can provide a list of redactions that you want to apply. If not provided, no redactions will be applied.
        :param keywords: the keywords to detect in the input. If you want to detect certain keywords in the input, you can provide a list of keywords that you want to detect. If not provided, no keyword detection will be run.
        :param access_policy: the access policy to run. This is the rego access policy that you can run. If not provided, no access policy will be applied.
        :param content_policy: the content policy to run. This is the rego content policy that you can run. If not provided, no content policy will be applied.
        """
        return self.scan_request(request=self.__build_scan_request(
            *messages,
            files=files,
            request_type=request_type,
            annotations=annotations,
            analyzers=analyzers,
            bypass_hash=bypass_hash,
            anonymization=anonymization,
            redactions=redactions,
            keywords=keywords,
            access_policy=access_policy,
            content_policy=content_policy,
        ))

    async def scan_async(
        self,
        *messages: str,
        files: Union[Sequence[Union[str,os.PathLike]], os.PathLike, str, None] = None,
        request_type: Union[ScanrequestType,str] = ScanrequestType.INPUT,
        annotations: Optional[Dict[str, str]] = None,
        analyzers: Optional[List[str]] = None,
        bypass_hash: Optional[str] = None,
        anonymization: Union[ScanrequestAnonymization, str, None] = None,
        redactions: Optional[List[str]] = None,
        keywords: Optional[List[str]] = None,
        access_policy: Optional[str] = None,
        content_policy: Optional[str] = None,
    ) -> Scanresponse:
        """
        scan_async() runs the provided messages (prompts) through the Acuvity detection engines and returns the results. Alternatively, you can run model output through the detection engines.
        Returns a Scanresponse object on success, and raises different exceptions on failure.

        This function allows to use and try different analyzers and make use of the redaction feature.
        You can also run access policies and content policies by passing them as parameters.

        :param messages: the messages to scan. These are the prompts that you want to scan. Required if no files or a direct request object are provided.
        :param files: the files to scan. These are the files that you want to scan. Required if no messages are provided. Can be used in addition to messages.        :param request_type: the type of the validation. This can be either ScanrequestType.INPUT or ScanrequestType.OUTPUT. Defaults to ScanrequestType.INPUT. Use ScanrequestType.OUTPUT if you want to run model output through the detection engines.
        :param annotations: the annotations to use. These are the annotations that you want to use. If not provided, no annotations will be used.
        :param analyzers: the analyzers to use. These are the analyzers that you want to use. If not provided, the internal default analyzers will be used. Use "+" to include an analyzer and "-" to exclude an analyzer. For example, ["+image-classifier", "-modality-detector"] will include the image classifier and exclude the modality detector. If any analyzer does not start with a '+' or '-', then the default analyzers will be replaced by whatever is provided. Call `list_analyzers()` and/or its variants to get a list of available analyzers.
        :param bypass_hash: the bypass hash to use. This is the hash that you want to use to bypass the detection engines. If not provided, no bypass hash will be used.
        :param anonymization: the anonymization to use. This is the anonymization that you want to use. If not provided, but the returned detections contain redactions, then the system will use the internal defaults for anonymization which is subject to change.
        :param redactions: the redactions to apply. If your want to redact certain parts of the returned detections, you can provide a list of redactions that you want to apply. If not provided, no redactions will be applied.
        :param keywords: the keywords to detect in the input. If you want to detect certain keywords in the input, you can provide a list of keywords that you want to detect. If not provided, no keyword detection will be run.
        :param access_policy: the access policy to run. This is the rego access policy that you can run. If not provided, no access policy will be applied.
        :param content_policy: the content policy to run. This is the rego content policy that you can run. If not provided, no content policy will be applied.
        """
        return await self.scan_request_async(request=self.__build_scan_request(
            *messages,
            files=files,
            request_type=request_type,
            annotations=annotations,
            analyzers=analyzers,
            bypass_hash=bypass_hash,
            anonymization=anonymization,
            redactions=redactions,
            keywords=keywords,
            access_policy=access_policy,
            content_policy=content_policy,
        ))

    def police(
        self,
        *messages: str,
        files: Union[Sequence[Union[str,os.PathLike]], os.PathLike, str, None] = None,
        request_type: Union[Type,str] = Type.INPUT,
        annotations: Optional[Dict[str, str]] = None,
        bypass_hash: Optional[str] = None,
        anonymization: Union[Anonymization, str, None] = None,
        provider: Optional[str] = None,
        user: Optional[Union[Policeexternaluser,Tuple[str, List[str]],Dict[str, Any]]] = None,
    ) -> Policeresponse:
        """
        police() runs the provided messages (prompts) through the Acuvity detection engines, applies policies, and returns the results. Alternatively, you can run model output through the detection engines.
        Returns a Policeresponse object on success, and raises different exceptions on failure.

        This function does **NOT** allow to use different analyzers or redactions as policies are being **managed** by the Acuvity backend.
        To configure different analyzers and redactions you must do so in the Acuvity backend.

        :param messages: the messages to scan. These are the prompts that you want to scan. Required if no files or a direct request object are provided.
        :param files: the files to scan. These are the files that you want to scan. Required if no messages are provided. Can be used in addition to messages.        :param request_type: the type of the validation. This can be either Type.INPUT or Type.OUTPUT. Defaults to Type.INPUT. Use Type.OUTPUT if you want to run model output through the detection engines.
        :param annotations: the annotations to use. These are the annotations that you want to use. If not provided, no annotations will be used.
        :param bypass_hash: the bypass hash to use. This is the hash that you want to use to bypass the detection engines. If not provided, no bypass hash will be used.
        :param anonymization: the anonymization to use. This is the anonymization that you want to use. If not provided, but the returned detections contain redactions, then the system will use the internal defaults for anonymization which is subject to change.
        :param provider: the provider to use. This is the provider name that you want to use for policy resolutions. If not provided, it will default to the principal name (the application itself).
        :param user: the user to use. This is the user name and their claims that you want to use.
        """
        return self.police_request(request=self.__build_police_request(
            *messages,
            files=files,
            request_type=request_type,
            annotations=annotations,
            bypass_hash=bypass_hash,
            anonymization=anonymization,
            provider=provider,
            user=user,
        ))

    async def police_async(
        self,
        *messages: str,
        files: Union[Sequence[Union[str,os.PathLike]], os.PathLike, str, None] = None,
        request_type: Union[Type,str] = Type.INPUT,
        annotations: Optional[Dict[str, str]] = None,
        bypass_hash: Optional[str] = None,
        anonymization: Union[Anonymization, str, None] = None,
        provider: Optional[str] = None,
        user: Optional[Union[Policeexternaluser,Tuple[str, List[str]],Dict[str, Any]]] = None,
    ) -> Policeresponse:
        """
        police_async() runs the provided messages (prompts) through the Acuvity detection engines, applies policies, and returns the results. Alternatively, you can run model output through the detection engines.
        Returns a Policeresponse object on success, and raises different exceptions on failure.

        This function does **NOT** allow to use different analyzers or redactions as policies are being **managed** by the Acuvity backend.
        To configure different analyzers and redactions you must do so in the Acuvity backend.

        :param messages: the messages to scan. These are the prompts that you want to scan. Required if no files or a direct request object are provided.
        :param files: the files to scan. These are the files that you want to scan. Required if no messages are provided. Can be used in addition to messages.        :param request_type: the type of the validation. This can be either Type.INPUT or Type.OUTPUT. Defaults to Type.INPUT. Use Type.OUTPUT if you want to run model output through the detection engines.
        :param annotations: the annotations to use. These are the annotations that you want to use. If not provided, no annotations will be used.
        :param bypass_hash: the bypass hash to use. This is the hash that you want to use to bypass the detection engines. If not provided, no bypass hash will be used.
        :param anonymization: the anonymization to use. This is the anonymization that you want to use. If not provided, but the returned detections contain redactions, then the system will use the internal defaults for anonymization which is subject to change.
        :param provider: the provider to use. This is the provider name that you want to use for policy resolutions. If not provided, it will default to the principal name (the application itself).
        :param user: the user to use. This is the user name and their claims that you want to use.
        """
        return await self.police_request_async(request=self.__build_police_request(
            *messages,
            files=files,
            request_type=request_type,
            annotations=annotations,
            bypass_hash=bypass_hash,
            anonymization=anonymization,
            provider=provider,
            user=user,
        ))

    def __build_police_request(
        self, # pylint: disable=unused-argument
        *messages: str,
        files: Union[Sequence[Union[str,os.PathLike]], os.PathLike, str, None] = None,
        request_type: Union[Type,str] = Type.INPUT,
        annotations: Optional[Dict[str, str]] = None,
        bypass_hash: Optional[str] = None,
        anonymization: Union[Anonymization, str, None] = None,
        provider: Optional[str] = None,
        user: Optional[Union[Policeexternaluser,Tuple[str, List[str]],Dict[str, Any]]] = None,
    ) -> Policerequest:
        request = Policerequest.model_construct()

        # messages must be strings
        for message in messages:
            if not isinstance(message, str):
                raise ValueError("messages must be strings")
        if len(messages) == 0 and files is None:
            raise ValueError("no messages and no files provided")
        if len(messages) > 0:
            request.messages = list(messages)

        # files must be a list of strings (or paths) or a single string (or path)
        extractions: List[Extractionrequest] = []
        if files is not None:
            process_files: List[Union[os.PathLike, str]] = []
            if isinstance(files, str):
                process_files.append(files)
            elif isinstance(files, os.PathLike):
                process_files.append(files)
            elif isinstance(files, Iterable):
                for file in files:
                    if not isinstance(file, str) and not isinstance(file, os.PathLike):
                        raise ValueError("files must be strings or paths")
                    process_files.append(file)
            else:
                raise ValueError("files must be strings or paths")
            for process_file in process_files:
                with open(process_file, 'rb') as opened_file:
                    file_content = opened_file.read()
                    # base64 encode the file content and then append
                    extractions.append(Extractionrequest(
                        data=base64.b64encode(file_content).decode("utf-8"),
                    ))
        if len(extractions) > 0:
            request.extractions = extractions

        # request_type must be either "Input" or "Output"
        if isinstance(request_type, Type):
            request.type = request_type
        elif isinstance(request_type, str):
            if request_type not in ("Input", "Output"):
                raise ValueError("request_type must be either 'Input' or 'Output'")
            request.type = Type(request_type)
        else:
            raise ValueError("type must be a 'str' or 'Type'")

        # annotations must be a dictionary of strings
        if annotations is not None:
            if not isinstance(annotations, dict):
                raise ValueError("annotations must be a dictionary")
            for key, value in annotations.items():
                if not isinstance(key, str) or not isinstance(value, str):
                    raise ValueError("annotations must be strings")
            request.annotations = annotations

        # bypass_hash must be a string
        if bypass_hash is not None:
            if not isinstance(bypass_hash, str):
                raise ValueError("bypass_hash must be a string")
            request.bypass_hash = bypass_hash

        # anonymization must be "FixedSize" or "VariableSize"
        if anonymization is not None:
            if isinstance(anonymization, Anonymization):
                request.anonymization = anonymization
            elif isinstance(anonymization, str):
                if anonymization not in ("FixedSize", "VariableSize"):
                    raise ValueError("anonymization must be 'FixedSize' or 'VariableSize'")
                request.anonymization = Anonymization(anonymization)
            else:
                raise ValueError("anonymization must be a 'str' or 'Anonymization'")

        # provider must be a string
        if provider is not None:
            if not isinstance(provider, str):
                raise ValueError("provider must be a string")
            request.provider = provider

        if user is not None:
            if isinstance(user, tuple):
                if len(user) != 2:
                    raise ValueError("user tuple must have exactly 2 elements to represent the name and claims")
                if not isinstance(user[0], str):
                    raise ValueError("user tuple first element must be a string to represent the name")
                if not isinstance(user[1], list):
                    raise ValueError("user tuple second element must be a list to represent the claims")
                for claim in user[1]:
                    if not isinstance(claim, str):
                        raise ValueError("user tuple second element must be a list of strings to represent the claims")
                request.user = Policeexternaluser(name=user[0], claims=user[1])
            elif isinstance(user, dict):
                name = user.get("name", None)
                if name is None:
                    raise ValueError("user dictionary must have a 'name' key to represent the name")
                if not isinstance(name, str):
                    raise ValueError("user dictionary 'name' key must be a string to represent the name")
                claims = user.get("claims", None)
                if claims is None:
                    raise ValueError("user dictionary must have a 'claims' key to represent the claims")
                if not isinstance(claims, list):
                    raise ValueError("user dictionary 'claims' key must be a list to represent the claims")
                for claim in claims:
                    if not isinstance(claim, str):
                        raise ValueError("user dictionary 'claims' key must be a list of strings to represent the claims")
                request.user = Policeexternaluser(name=name, claims=claims)
            elif isinstance(user, Policeexternaluser):
                request.user = user
            else:
                raise ValueError("user must be a tuple, dictionary or Policeexternaluser object")

        return request

    def __build_scan_request(
        self,
        *messages: str,
        files: Union[Sequence[Union[str,os.PathLike]], os.PathLike, str, None] = None,
        request_type: Union[ScanrequestType,str] = ScanrequestType.INPUT,
        annotations: Optional[Dict[str, str]] = None,
        analyzers: Optional[List[str]] = None,
        bypass_hash: Optional[str] = None,
        anonymization: Union[ScanrequestAnonymization, str, None] = None,
        redactions: Optional[List[str]] = None,
        keywords: Optional[List[str]] = None,
        access_policy: Optional[str] = None,
        content_policy: Optional[str] = None,
    ) -> Scanrequest:
        request = Scanrequest.model_construct()

        # messages must be strings
        for message in messages:
            if not isinstance(message, str):
                raise ValueError("messages must be strings")
        if len(messages) == 0 and files is None:
            raise ValueError("no messages and no files provided")
        if len(messages) > 0:
            request.messages = list(messages)

        # files must be a list of strings (or paths) or a single string (or path)
        extractions: List[Extractionrequest] = []
        if files is not None:
            process_files: List[Union[os.PathLike, str]] = []
            if isinstance(files, str):
                process_files.append(files)
            elif isinstance(files, os.PathLike):
                process_files.append(files)
            elif isinstance(files, Iterable):
                for file in files:
                    if not isinstance(file, str) and not isinstance(file, os.PathLike):
                        raise ValueError("files must be strings or paths")
                    process_files.append(file)
            else:
                raise ValueError("files must be strings or paths")
            for process_file in process_files:
                with open(process_file, 'rb') as opened_file:
                    file_content = opened_file.read()
                    # base64 encode the file content and then append
                    extractions.append(Extractionrequest(
                        data=base64.b64encode(file_content).decode("utf-8"),
                    ))
        if len(extractions) > 0:
            request.extractions = extractions

        # request_type must be either "Input" or "Output"
        if isinstance(request_type, ScanrequestType):
            request.type = request_type
        elif isinstance(request_type, str):
            if request_type not in ("Input", "Output"):
                raise ValueError("request_type must be either 'Input' or 'Output'")
            request.type = ScanrequestType(request_type)
        else:
            raise ValueError("type must be a 'str' or 'Type'")

        # annotations must be a dictionary of strings
        if annotations is not None:
            if not isinstance(annotations, dict):
                raise ValueError("annotations must be a dictionary")
            for key, value in annotations.items():
                if not isinstance(key, str) or not isinstance(value, str):
                    raise ValueError("annotations must be strings")
            request.annotations = annotations

        # analyzers must be a list of strings
        if analyzers is not None:
            if not isinstance(analyzers, List):
                raise ValueError("analyzers must be a list")
            analyzers_list = self.list_analyzer_groups() + self.list_analyzer_names()
            for analyzer in analyzers:
                if not isinstance(analyzer, str):
                    raise ValueError("analyzers must be strings")
                if analyzer.startswith(("+", "-")):
                    analyzer = analyzer[1:]
                if analyzer not in analyzers_list:
                    raise ValueError(f"analyzer '{analyzer}' is not in list of analyzer groups or analyzers: {analyzers_list}")
            request.analyzers = analyzers

        # bypass_hash must be a string
        if bypass_hash is not None:
            if not isinstance(bypass_hash, str):
                raise ValueError("bypass_hash must be a string")
            request.bypass_hash = bypass_hash

        # anonymization must be "FixedSize" or "VariableSize"
        if anonymization is not None:
            if isinstance(anonymization, ScanrequestAnonymization):
                request.anonymization = anonymization
            elif isinstance(anonymization, str):
                if anonymization not in ("FixedSize", "VariableSize"):
                    raise ValueError("anonymization must be 'FixedSize' or 'VariableSize'")
                request.anonymization = ScanrequestAnonymization(anonymization)
            else:
                raise ValueError("anonymization must be a 'str' or 'Anonymization'")

        # redactions must be a list of strings
        if redactions is not None:
            if not isinstance(redactions, List):
                raise ValueError("redactions must be a list")
            for redaction in redactions:
                if not isinstance(redaction, str):
                    raise ValueError("redactions must be strings")
            request.redactions = redactions

        # keywords must be a list of strings
        if keywords is not None:
            if not isinstance(keywords, List):
                raise ValueError("keywords must be a list")
            for keyword in keywords:
                if not isinstance(keyword, str):
                    raise ValueError("keywords must be strings")
            request.keywords = keywords

        # local access policy
        if access_policy is not None:
            if not isinstance(access_policy, str):
                raise ValueError("access_policy must be a string")
            request.access_policy = access_policy

        # local content policy
        if content_policy is not None:
            if not isinstance(content_policy, str):
                raise ValueError("content_policy must be a string")
            request.content_policy = content_policy

        return request



"""Code originally generated by Speakeasy (https://speakeasy.com)."""

from ._version import __title__, __version__

# pylint: disable=unused-import
import acuvity.apexextend

# pylint: disable=unused-import
import acuvity.sdkextend

from .sdk import *
from .sdkconfiguration import *
from .models import *


VERSION: str = __version__



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .extractionrequest import Extractionrequest, ExtractionrequestTypedDict
from acuvity.types import BaseModel
from enum import Enum
import pydantic
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ScanrequestAnonymization(str, Enum):
    r"""How to anonymize the data. If deanonymize is true, then VariablSize is required."""

    FIXED_SIZE = "FixedSize"
    VARIABLE_SIZE = "VariableSize"


class ScanrequestType(str, Enum):
    r"""The type of text."""

    INPUT = "Input"
    OUTPUT = "Output"


class ScanrequestTypedDict(TypedDict):
    r"""This is a scan request."""

    access_policy: NotRequired[str]
    r"""AccessPolicy allows to pass optional Rego access policy. If not set,
    The action is always Allow,
    If it is set, it will be run, and the final decision will be computed based
    on that policy.
    If the rego code does not start with package main, then the needed
    classic package definition and  acuvity imports will be added
    automatically.
    If the code starts with package main, then everything remains untouched.
    """
    analyzers: NotRequired[List[str]]
    r"""The name of the analyzers to run by the pipelines. If empty, it means all of
    them. Possible values are PIIs, Secrets, Topics, Confidentiality, Exploits.
    """
    annotations: NotRequired[Dict[str, str]]
    r"""Annotations attached to the extraction."""
    anonymization: NotRequired[ScanrequestAnonymization]
    r"""How to anonymize the data. If deanonymize is true, then VariablSize is required."""
    bypass_hash: NotRequired[str]
    r"""In the case of a contentPolicy that asks for a confirmation, this is the
    hash you must send back to bypass the block. This is only useful when a
    content policy has been set or is evaluated remotely.
    """
    content_policy: NotRequired[str]
    r"""ContentPolicy allows to pass optional Rego content policy. If not set,
    The action is always Allow, and there cannot be any alerts raised etc
    If it is set, it will be run, and the final decision will be computed based
    on that policy.
    If the rego code does not start with package main, then the needed
    classic package definition and  acuvity imports will be added
    automatically.
    If the code starts with package main, then everything remains untouched.
    """
    extractions: NotRequired[List[ExtractionrequestTypedDict]]
    r"""The extractions to request."""
    keywords: NotRequired[List[str]]
    r"""The keywords found during classification."""
    messages: NotRequired[List[str]]
    r"""Messages to process and provide detections for. Use data in extractions for
    processing binary data.
    """
    minimal_logging: NotRequired[bool]
    r"""If true, the system will not log the contents that were scanned."""
    redactions: NotRequired[List[str]]
    r"""The redactions to perform if they are detected."""
    type: NotRequired[ScanrequestType]
    r"""The type of text."""


class Scanrequest(BaseModel):
    r"""This is a scan request."""

    access_policy: Annotated[Optional[str], pydantic.Field(alias="accessPolicy")] = None
    r"""AccessPolicy allows to pass optional Rego access policy. If not set,
    The action is always Allow,
    If it is set, it will be run, and the final decision will be computed based
    on that policy.
    If the rego code does not start with package main, then the needed
    classic package definition and  acuvity imports will be added
    automatically.
    If the code starts with package main, then everything remains untouched.
    """

    analyzers: Optional[List[str]] = None
    r"""The name of the analyzers to run by the pipelines. If empty, it means all of
    them. Possible values are PIIs, Secrets, Topics, Confidentiality, Exploits.
    """

    annotations: Optional[Dict[str, str]] = None
    r"""Annotations attached to the extraction."""

    anonymization: Optional[ScanrequestAnonymization] = (
        ScanrequestAnonymization.FIXED_SIZE
    )
    r"""How to anonymize the data. If deanonymize is true, then VariablSize is required."""

    bypass_hash: Annotated[Optional[str], pydantic.Field(alias="bypassHash")] = None
    r"""In the case of a contentPolicy that asks for a confirmation, this is the
    hash you must send back to bypass the block. This is only useful when a
    content policy has been set or is evaluated remotely.
    """

    content_policy: Annotated[Optional[str], pydantic.Field(alias="contentPolicy")] = (
        None
    )
    r"""ContentPolicy allows to pass optional Rego content policy. If not set,
    The action is always Allow, and there cannot be any alerts raised etc
    If it is set, it will be run, and the final decision will be computed based
    on that policy.
    If the rego code does not start with package main, then the needed
    classic package definition and  acuvity imports will be added
    automatically.
    If the code starts with package main, then everything remains untouched.
    """

    extractions: Optional[List[Extractionrequest]] = None
    r"""The extractions to request."""

    keywords: Optional[List[str]] = None
    r"""The keywords found during classification."""

    messages: Optional[List[str]] = None
    r"""Messages to process and provide detections for. Use data in extractions for
    processing binary data.
    """

    minimal_logging: Annotated[
        Optional[bool], pydantic.Field(alias="minimalLogging")
    ] = None
    r"""If true, the system will not log the contents that were scanned."""

    redactions: Optional[List[str]] = None
    r"""The redactions to perform if they are detected."""

    type: Optional[ScanrequestType] = None
    r"""The type of text."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from pydantic import ConfigDict, model_serializer
from pydantic import BaseModel as PydanticBaseModel
from typing import TYPE_CHECKING, Literal, Optional, TypeVar, Union, NewType
from typing_extensions import TypeAliasType, TypeAlias


class BaseModel(PydanticBaseModel):
    model_config = ConfigDict(
        populate_by_name=True, arbitrary_types_allowed=True, protected_namespaces=()
    )


class Unset(BaseModel):
    @model_serializer(mode="plain")
    def serialize_model(self):
        return UNSET_SENTINEL

    def __bool__(self) -> Literal[False]:
        return False


UNSET = Unset()
UNSET_SENTINEL = "~?~unset~?~sentinel~?~"


T = TypeVar("T")
if TYPE_CHECKING:
    Nullable: TypeAlias = Union[T, None]
    OptionalNullable: TypeAlias = Union[Optional[Nullable[T]], Unset]
else:
    Nullable = TypeAliasType("Nullable", Union[T, None], type_params=(T,))
    OptionalNullable = TypeAliasType(
        "OptionalNullable", Union[Optional[Nullable[T]], Unset], type_params=(T,)
    )

UnrecognizedInt = NewType("UnrecognizedInt", int)
UnrecognizedStr = NewType("UnrecognizedStr", str)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .extractionrequest import Extractionrequest, ExtractionrequestTypedDict
from .policeexternaluser import Policeexternaluser, PoliceexternaluserTypedDict
from acuvity.types import BaseModel
from enum import Enum
import pydantic
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class Anonymization(str, Enum):
    r"""How to anonymize the data. If deanonymize is true, then VariablSize is required."""

    FIXED_SIZE = "FixedSize"
    VARIABLE_SIZE = "VariableSize"


class Type(str, Enum):
    r"""The type of text."""

    INPUT = "Input"
    OUTPUT = "Output"


class PolicerequestTypedDict(TypedDict):
    r"""This is a scan and police request."""

    annotations: NotRequired[Dict[str, str]]
    r"""Annotations attached to the extraction."""
    anonymization: NotRequired[Anonymization]
    r"""How to anonymize the data. If deanonymize is true, then VariablSize is required."""
    bypass_hash: NotRequired[str]
    r"""In the case of a contentPolicy that asks for a confirmation, this is the
    hash you must send back to bypass the block. This is only useful when a
    content policy has been set or is evaluated remotely.
    """
    extractions: NotRequired[List[ExtractionrequestTypedDict]]
    r"""The extractions to request."""
    messages: NotRequired[List[str]]
    r"""Messages to process and provide detections for. Use data in extractions for
    processing binary data.
    """
    provider: NotRequired[str]
    r"""The name of the provider to use for policy resolutions. If not set, it will
    default to the principal name (the application itself).
    """
    type: NotRequired[Type]
    r"""The type of text."""
    user: NotRequired[PoliceexternaluserTypedDict]
    r"""PoliceExternalUser holds the information about the remote user for a
    PoliceRequest.
    """


class Policerequest(BaseModel):
    r"""This is a scan and police request."""

    annotations: Optional[Dict[str, str]] = None
    r"""Annotations attached to the extraction."""

    anonymization: Optional[Anonymization] = Anonymization.FIXED_SIZE
    r"""How to anonymize the data. If deanonymize is true, then VariablSize is required."""

    bypass_hash: Annotated[Optional[str], pydantic.Field(alias="bypassHash")] = None
    r"""In the case of a contentPolicy that asks for a confirmation, this is the
    hash you must send back to bypass the block. This is only useful when a
    content policy has been set or is evaluated remotely.
    """

    extractions: Optional[List[Extractionrequest]] = None
    r"""The extractions to request."""

    messages: Optional[List[str]] = None
    r"""Messages to process and provide detections for. Use data in extractions for
    processing binary data.
    """

    provider: Optional[str] = None
    r"""The name of the provider to use for policy resolutions. If not set, it will
    default to the principal name (the application itself).
    """

    type: Optional[Type] = None
    r"""The type of text."""

    user: Optional[Policeexternaluser] = None
    r"""PoliceExternalUser holds the information about the remote user for a
    PoliceRequest.
    """



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .alertevent import Alertevent, AlerteventTypedDict
from .extraction import Extraction, ExtractionTypedDict
from .latency import Latency, LatencyTypedDict
from .principal import Principal, PrincipalTypedDict
from acuvity.types import BaseModel
from datetime import datetime
from enum import Enum
import pydantic
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ScanresponseDecision(str, Enum):
    r"""Tell what was the decision about the data."""

    DENY = "Deny"
    ALLOW = "Allow"
    ASK = "Ask"
    BYPASSED = "Bypassed"
    FORBIDDEN_USER = "ForbiddenUser"


class ScanresponseType(str, Enum):
    r"""The type of text."""

    INPUT = "Input"
    OUTPUT = "Output"


class ScanresponseTypedDict(TypedDict):
    r"""This is a scan response."""

    principal: PrincipalTypedDict
    r"""Describe the principal."""
    id: NotRequired[str]
    r"""ID is the identifier of the object."""
    alerts: NotRequired[List[AlerteventTypedDict]]
    r"""List of alerts that got raised during the policy resolution."""
    annotations: NotRequired[Dict[str, str]]
    r"""Annotations attached to the log."""
    client: NotRequired[str]
    r"""The client used to send the request."""
    client_version: NotRequired[str]
    r"""The version of the client used to send the request."""
    decision: NotRequired[ScanresponseDecision]
    r"""Tell what was the decision about the data."""
    extractions: NotRequired[List[ExtractionTypedDict]]
    r"""The extractions to log."""
    hash: NotRequired[str]
    r"""The hash of the input."""
    latency: NotRequired[LatencyTypedDict]
    r"""Holds information about latencies introduced by Apex."""
    namespace: NotRequired[str]
    r"""The namespace of the object."""
    pipeline_name: NotRequired[str]
    r"""The name of the particular pipeline that extracted the text."""
    provider: NotRequired[str]
    r"""the provider to use."""
    reasons: NotRequired[List[str]]
    r"""The various reasons returned by the policy engine."""
    time: NotRequired[datetime]
    r"""Set the time of the message request."""
    type: NotRequired[ScanresponseType]
    r"""The type of text."""


class Scanresponse(BaseModel):
    r"""This is a scan response."""

    principal: Principal
    r"""Describe the principal."""

    id: Annotated[Optional[str], pydantic.Field(alias="ID")] = None
    r"""ID is the identifier of the object."""

    alerts: Optional[List[Alertevent]] = None
    r"""List of alerts that got raised during the policy resolution."""

    annotations: Optional[Dict[str, str]] = None
    r"""Annotations attached to the log."""

    client: Optional[str] = None
    r"""The client used to send the request."""

    client_version: Annotated[Optional[str], pydantic.Field(alias="clientVersion")] = (
        None
    )
    r"""The version of the client used to send the request."""

    decision: Optional[ScanresponseDecision] = None
    r"""Tell what was the decision about the data."""

    extractions: Optional[List[Extraction]] = None
    r"""The extractions to log."""

    hash: Optional[str] = None
    r"""The hash of the input."""

    latency: Optional[Latency] = None
    r"""Holds information about latencies introduced by Apex."""

    namespace: Optional[str] = None
    r"""The namespace of the object."""

    pipeline_name: Annotated[Optional[str], pydantic.Field(alias="pipelineName")] = None
    r"""The name of the particular pipeline that extracted the text."""

    provider: Optional[str] = None
    r"""the provider to use."""

    reasons: Optional[List[str]] = None
    r"""The various reasons returned by the policy engine."""

    time: Optional[datetime] = None
    r"""Set the time of the message request."""

    type: Optional[ScanresponseType] = None
    r"""The type of text."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .modality import Modality, ModalityTypedDict
from .textualdetection import Textualdetection, TextualdetectionTypedDict
from acuvity.types import BaseModel
import pydantic
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ExtractionTypedDict(TypedDict):
    r"""Represents the extracted information to log."""

    pi_is: NotRequired[Dict[str, float]]
    r"""The PIIs found during classification."""
    annotations: NotRequired[Dict[str, str]]
    r"""Annotations attached to the extraction."""
    categories: NotRequired[List[ModalityTypedDict]]
    r"""The categories are remapping of the modalities in a more human friendly way."""
    confidentiality: NotRequired[float]
    r"""The level of general confidentiality of the input."""
    data: NotRequired[str]
    r"""The data extracted."""
    detections: NotRequired[List[TextualdetectionTypedDict]]
    r"""The detections found while applying policies."""
    exploits: NotRequired[Dict[str, float]]
    r"""The various exploits attempts."""
    hash: NotRequired[str]
    r"""The hash of the extraction."""
    intent: NotRequired[Dict[str, float]]
    r"""The estimated intent embodied into the text."""
    internal: NotRequired[bool]
    r"""If true, this extraction is for internal use only. This can be used by agentic
    systems to mark an extraction as internal only as opposed to user facing.
    """
    keywords: NotRequired[Dict[str, float]]
    r"""The keywords found during classification."""
    label: NotRequired[str]
    r"""A means of distinguishing what was extracted, such as prompt, input file or
    code.
    """
    languages: NotRequired[Dict[str, float]]
    r"""The language of the classification."""
    modalities: NotRequired[List[ModalityTypedDict]]
    r"""The modalities of data detected in the data."""
    relevance: NotRequired[float]
    r"""The level of general organization relevance of the input."""
    secrets: NotRequired[Dict[str, float]]
    r"""The secrets found during classification."""
    topics: NotRequired[Dict[str, float]]
    r"""The topic of the classification."""


class Extraction(BaseModel):
    r"""Represents the extracted information to log."""

    pi_is: Annotated[Optional[Dict[str, float]], pydantic.Field(alias="PIIs")] = None
    r"""The PIIs found during classification."""

    annotations: Optional[Dict[str, str]] = None
    r"""Annotations attached to the extraction."""

    categories: Optional[List[Modality]] = None
    r"""The categories are remapping of the modalities in a more human friendly way."""

    confidentiality: Optional[float] = None
    r"""The level of general confidentiality of the input."""

    data: Optional[str] = None
    r"""The data extracted."""

    detections: Optional[List[Textualdetection]] = None
    r"""The detections found while applying policies."""

    exploits: Optional[Dict[str, float]] = None
    r"""The various exploits attempts."""

    hash: Optional[str] = None
    r"""The hash of the extraction."""

    intent: Optional[Dict[str, float]] = None
    r"""The estimated intent embodied into the text."""

    internal: Optional[bool] = None
    r"""If true, this extraction is for internal use only. This can be used by agentic
    systems to mark an extraction as internal only as opposed to user facing.
    """

    keywords: Optional[Dict[str, float]] = None
    r"""The keywords found during classification."""

    label: Optional[str] = None
    r"""A means of distinguishing what was extracted, such as prompt, input file or
    code.
    """

    languages: Optional[Dict[str, float]] = None
    r"""The language of the classification."""

    modalities: Optional[List[Modality]] = None
    r"""The modalities of data detected in the data."""

    relevance: Optional[float] = None
    r"""The level of general organization relevance of the input."""

    secrets: Optional[Dict[str, float]] = None
    r"""The secrets found during classification."""

    topics: Optional[Dict[str, float]] = None
    r"""The topic of the classification."""
